import { DefaultDraftBlockRenderMap, EditorState, RichUtils, } from 'draft-js';
import * as Immutable from 'immutable';
import { createHorizontalRuleAtomicComponentEntry, createImageAtomicComponentEntry, } from './@atomics';
import { handleBlockSplitting, handleCodePasting, handleInlineStyleOverriding, handleMultilineBlockReturn, handleTabIndent, } from './@behaviors';
import { checkableListItemRenderEntry, createCheckableListItemEntry, } from './@blocks';
import { createCodeDecorator, createLinkDecorator, } from './@decorators';
import { createBlockquoteFeature, createBoldFeature, createCheckableListItemFeature, createCodeBlockFeature, createCodeFeature, createHeaderFeature, createHorizontalRuleFeature, createImageFeature, createItalicFeature, createLinkFeature, createListFeature, createStrikethroughFeature, } from './@features';
export class FluentMarkdownPlugin {
    constructor({ block = true, link: linkOptions = {}, indent: indentOptions, }) {
        // Using IME may end up unexpected selection change after applying a feature.
        // E.g.: `*test*|test` could end up with `<i>test</i>te|st`, while the
        // correct result should be `<i>test</i>|test`. The feature selection lock
        // hack sets a timer and simply ignored selection changes immediately after a
        // feature being applied.
        this.featureSelectionLocked = false;
        this.onTab = (event, { getEditorState, setEditorState }) => {
            let editorState = getEditorState();
            let nextEditorState = handleTabIndent(editorState, event, this.indentOptions);
            if (nextEditorState === editorState) {
                nextEditorState = RichUtils.onTab(event, editorState, 4);
            }
            if (nextEditorState !== editorState) {
                setEditorState(nextEditorState);
                return 'handled';
            }
            else {
                return 'not-handled';
            }
        };
        this.blockRendererFn = (block, pluginFunctions) => {
            let descriptor;
            let blockType = block.getType();
            if (blockType === 'atomic') {
                let data = block.getData();
                let type = data.get('type');
                descriptor = Object.assign({}, this.atomicDescriptorMap.get(type), { editable: false });
            }
            else {
                let builder = this.blockDescriptorBuilderMap.get(blockType);
                if (builder) {
                    descriptor = builder(pluginFunctions);
                }
            }
            return descriptor;
        };
        this.handleBeforeInput = (input, editorState, { setEditorState }) => {
            let nextEditorState = this.triggerFeature(input, editorState);
            if (nextEditorState !== editorState) {
                setEditorState(nextEditorState);
                clearTimeout(this.featureSelectionLockTimer);
                this.featureSelectionLocked = true;
                this.featureSelectionLockTimer = setTimeout(() => (this.featureSelectionLocked = false));
                return 'handled';
            }
            else {
                return 'not-handled';
            }
        };
        this.handleReturn = (event, editorState, { setEditorState }) => {
            let nextEditorState = handleMultilineBlockReturn(event, editorState);
            if (nextEditorState !== editorState) {
                setEditorState(nextEditorState);
                return 'handled';
            }
            else {
                return 'not-handled';
            }
        };
        this.handleKeyCommand = (command, editorState, { setEditorState }) => {
            let nextEditorState;
            switch (command) {
                case 'split-block':
                    nextEditorState = handleBlockSplitting(editorState);
                    break;
            }
            if (nextEditorState && nextEditorState !== editorState) {
                setEditorState(nextEditorState);
                return 'handled';
            }
            else {
                return 'not-handled';
            }
        };
        this.handlePastedText = (text, _html, editorState, { setEditorState }) => {
            let nextEditorState = handleCodePasting(editorState, text);
            if (nextEditorState && nextEditorState !== editorState) {
                setEditorState(nextEditorState);
                return 'handled';
            }
            else {
                return 'not-handled';
            }
        };
        this.onChange = (editorState) => {
            if (this.featureSelectionLocked) {
                let content = editorState.getCurrentContent();
                editorState = EditorState.forceSelection(editorState, content.getSelectionAfter());
            }
            return handleInlineStyleOverriding(editorState);
        };
        this.decorators = [createCodeDecorator(), createLinkDecorator(linkOptions)];
        let atomicComponentEntries = [];
        let blockBuilderEntries = [];
        let blockRenderMap;
        let features = [
            createBoldFeature(),
            createItalicFeature(),
            createStrikethroughFeature(),
            createCodeFeature(),
            createLinkFeature(),
        ];
        if (block) {
            atomicComponentEntries.push(createImageAtomicComponentEntry(), createHorizontalRuleAtomicComponentEntry());
            blockBuilderEntries.push(createCheckableListItemEntry());
            blockRenderMap = DefaultDraftBlockRenderMap.merge(Immutable.Map([checkableListItemRenderEntry]));
            features.push(createImageFeature(), createHeaderFeature(), createListFeature(), createBlockquoteFeature(), createCodeBlockFeature(), createHorizontalRuleFeature(), createCheckableListItemFeature());
        }
        else {
            blockRenderMap = DefaultDraftBlockRenderMap;
        }
        this.atomicDescriptorMap = new Map(atomicComponentEntries);
        this.blockDescriptorBuilderMap = new Map(blockBuilderEntries);
        this.blockRenderMap = blockRenderMap;
        this.features = features;
        this.indentOptions = indentOptions;
    }
    triggerFeature(input, editorState) {
        for (let feature of this.features) {
            let nextEditorState = feature(input, editorState);
            if (nextEditorState) {
                return nextEditorState;
            }
        }
        return editorState;
    }
}
export function createFluentMarkdownPlugin(options = {}) {
    return new FluentMarkdownPlugin(options);
}
//# sourceMappingURL=plugin.js.map